/* Implementiamo prima una routine che consente di effettuare il calcolo dei p-value
  e dei p-value-like, da utilizzare nella routine NPC (tipo = pval_like o pval) */
%macro comp_pval(type,ds,var);
	proc datasets nolist;
		delete result_conteggio;
	run;

	%if &type = pval_like %then %let eoc = (&B + 1);
	%if &type = pval %then %let eoc = 1;
	
	%do scanner=1 %to &eoc;
		/* A ogni ripetizione del ciclo si punta alla osservazione
	      i-esima e la si fa diventare una MV soglia */
		data _NULL_;
			ii = &scanner;
			set &ds point=ii;
			call symput('soglia',&var);
			stop;
		run;

		/* Fissata la soglia si scorre tutto il ds e si contano i superi.
		  Alla ultima iterazione del datastep il conteggio, che conterr
		  il numero tot di superi, viene trasformato in MV */
		data _NULL_;
			set &ds;
			if _N_^ = &scanner and &var >= &soglia then count+1;
			call symput('n_superi',count);
		run;

		data tmp;
			scanner=&scanner; 
			soglia = &soglia;
			n_superi  = &n_superi; 
			pval=(1/2+&n_superi)/(&B+1);
		run;
		proc append data=tmp base=result_conteggio; run;
	%end;
%mend;

/* La procedura NPC viene implementata usando la statistica test di Gray come riferimento */
%macro NPC(ds,tempo=,evento=,gruppo=);
	/* Eliminiamo tre dataset che conterranno le statistiche test di Gray
  	  per ciascuno dei tre eventi */
		proc datasets library=work nolist;
			delete chi_0 chi_1 chi_2;
		run;

	/* Calcolo della statistica test di Gray per la censura.
      In questo caso il censoring viene settato a 3 (valore fittizio)
      mentre lo evento principale e il competitivo sono trattati come rischi competitivi */
	%if &versione_sas = _9_4 %then %do;
		proc lifetest data = &ds plots = &grafici;
			time &tempo*&evento(3) / eventcode = 0;
			strata &gruppo;
			ods output GrayTest = output_G_NPC(keep= failcode chisq);
		run;
		proc append base = chi_0 data = output_G_NPC; run;
	%end;
	%else %if &versione_sas = _9_3 %then %do;
		%CIF(data=&ds, time=&tempo, status=&evento, group=&gruppo, event=0, censored=3, alpha=.05, options=NOPLOT)
		data testresult; set testresult(keep= test_stat);  Failcode = 0; run;
		proc append base = chi_0 data = testresult( rename = (test_stat = ChiSq) ); run;
	%end;
		
	/* Calcolo della statistica Gray per lo evento principale e il rischio competitivo.
	  In questo caso il censoring  settato ovviamente in corrispondenza dello 0 */
	%if &versione_sas = _9_4 %then %do;
		proc lifetest data = &ds plots = &grafici;
			time &tempo*&evento(0) / eventcode = 1 2;
			strata &gruppo;
			ods output GrayTest = output_G_NPC(keep= failcode chisq);
		run;
		proc append base = chi_1 data = output_G_NPC( where = (failcode=1) ); run;
		proc append base = chi_2 data = output_G_NPC( where = (failcode=2) ); run;
	%end;
	%else %if &versione_sas = _9_3 %then %do;
		%CIF(data=&ds, time=&tempo, status=&evento, group=&gruppo, event=1, censored=0, alpha=.05, options=NOPLOT)
		data testresult; set testresult(keep= test_stat); Failcode = 1; run;
		proc append base = chi_1 data = testresult( rename = (test_stat = ChiSq) ); run;
		%CIF(data=&ds, time=&tempo, status=&evento, group=&gruppo, event=2, censored=0, alpha=.05, options=NOPLOT)
		data testresult; set testresult(keep= test_stat); Failcode = 2; run;
		proc append base = chi_2 data = testresult( rename = (test_stat = ChiSq) ); run;
	%end;

	/* Adesso questa routine viene ripetuta permutando il dataset B volte */
	%do P = 1 %to &B;
		DM 'clear log';
		
		/* Per permutare la appartenenza ai gruppi delle varie osservazioni,
          alla configurazione iniziale dei gruppi affianchiamo una variabile
          con osservazioni simulate da una uniforme e ordiniamo il dataset cos
          ottenuto rispetto ai numeri distribuiti uniformemente */
		data da_permutare;
			set &ds( keep = &gruppo );
			rename &gruppo = &gruppo._perm&P;
			/*call streaminit(123+&P);*/
			ordine = rand("uniform");
		run;
		proc sort data = da_permutare out = gruppi_permutati( drop = ordine );
			by ordine;
		run;

		data dati_permutati( drop = &gruppo );
			set &ds;
			merge gruppi_permutati &ds;
		run;
		%if &versione_sas = _9_4 %then %do;
			proc lifetest data = dati_permutati plots = &grafici;
				time &tempo*&evento(3) / eventcode = 0;
				strata &gruppo._perm&P;
				ods output GrayTest = output_G_NPC(keep= failcode chisq);
			run;
			proc append base = chi_0 data = output_G_NPC; run;
			proc lifetest data = dati_permutati plots = &grafici;
				time &tempo*&evento(0) / eventcode = 1 2;
				strata &gruppo._perm&P;
				ods output GrayTest = output_G_NPC(keep= failcode chisq);
			run;
			proc append base = chi_1 data = output_G_NPC( where = (failcode=1) ); run;
			proc append base = chi_2 data = output_G_NPC( where = (failcode=2) ); run;
		%end;
		%else %if &versione_sas = _9_3 %then %do;
			%CIF(data=dati_permutati, time=&tempo, status=&evento, group=&gruppo._perm&P, event=0, censored=3, alpha=.05, options=NOPLOT)
			data testresult; set testresult(keep= test_stat);  Failcode = 0; run;
			proc append base = chi_0 data = testresult( rename = (test_stat = ChiSq) ); run;
			%CIF(data=dati_permutati, time=&tempo, status=&evento, group=&gruppo._perm&P, event=1, censored=0, alpha=.05, options=NOPLOT)
			data testresult; set testresult(keep= test_stat);  Failcode = 1; run;
			proc append base = chi_1 data = testresult( rename = (test_stat = ChiSq) ); run;
			%CIF(data=dati_permutati, time=&tempo, status=&evento, group=&gruppo._perm&P, event=2, censored=0, alpha=.05, options=NOPLOT)
			data testresult; set testresult(keep= test_stat);  Failcode = 2; run;
			proc append base = chi_2 data = testresult( rename = (test_stat = ChiSq) ); run;

		%end;
			
	%end;

	/* Calcoliamo i p-value e i p-value-like per i tre eventi */
	%comp_pval(pval_like,chi_0,chisq)
	data chi_0; merge chi_0 result_conteggio; run;

	%comp_pval(pval_like,chi_1,chisq)
	data chi_1; merge chi_1 result_conteggio; run;

	%comp_pval(pval_like,chi_2,chisq)
	data chi_2; merge chi_2 result_conteggio; run;

	/* Uniamo i p-value-like per i tre eventi per poter calcolare
      le funzioni di combinazione di Tippet e Fisher */
	data pval_0(keep = PG0);
		set chi_0(rename = (pval = PG0));
	run;
	data pval_1(keep = PG1);
		set chi_1(rename = (pval = PG1));
	run;
	data pval_2(keep = PG2);
		set chi_2(rename = (pval = PG2));
	run;
	data output_NPC;
		merge pval_0 pval_1 pval_2;
		Tippet_012 = min(PG0,PG1,PG2);
		Fisher_012 = - log(PG0) - log(PG1) - log(PG2);
		Tippet_12 = min(PG1,PG2);
		Fisher_12 = - log(PG1) - log(PG2);
	run;
	/* Combiniamo i Fisher e i Tippet sia considerando tutti gli eventi
	  che prendendo in considerazione solo 1 e 2 */
	%comp_pval(pval,output_npc,Fisher_012)
	data f012;
		keep PF_012;
		set result_conteggio;
		PF_012=pval;
	run;

	%comp_pval(pval,output_npc,Tippet_012)
	data t012;
		keep PT_012;
		set result_conteggio;
		PT_012=1 - pval; /* per i Tippet i valori devono essere minori o uguali della soglia */ 
	run;

	%comp_pval(pval,output_npc,Fisher_12)
	data f12;
		keep PF_12;
		set result_conteggio;
		PF_12=pval;
	run;

	%comp_pval(pval,output_npc,Tippet_12)
	data t12;
		keep PT_12;
		set result_conteggio;
		PT_12=1 - pval; /* per i Tippet i valori devono essere minori o uguali della soglia */
	run;

	data pgs;
	keep pg0 pg1 pg2;
		set output_npc;
		if _n_=1 then output;
	run;

	data final_result_NPC;
		merge pgs f012 t012 f12 t12;
	run;

%mend NPC;

