*Il path in cui salvare i risultati della simulazione dipende dall'utente
 che esegue la simulazione tramite la macro %settings;
libname results "&path_risultati";

* Nel seguente dataset sono presenti i parametri relativi agli scenari
  possibili che si possono simulare (tale formato va bene soprattutto per cause-specific hazard costanti). 
  La scelta di quali e quanti scenari si vogliono mandare in esecuzione vengono 
  impostate nella macro settings tramite le variabili &lista_scenari e &nscenari.
  t_admin rappresenta il tempo oltre il quale scatta il censoring amministrativo.
  Il censoring viene sempre simulato come una variabile uniforme di estremi 0 e theta_c;
data scenari;
	input id_scen t_admin n_a n_b lambda_A1 lambda_A2 lambda_B1 lambda_B2 theta_c eol $;
	datalines;
1 5 25 20 1 1 1 1.2 5 x
2 5 25 20 1 1 1 1.6 5 x
3 5 25 20 1 1 1.6 1 5 x
4 5 25 20 1 1 1.6 1.2 5 x
5 5 25 20 1 1 1.6 1.6 5 x
6 5 25 20 1 1 1 1 5 x
;
run;
data scenari;
	set scenari;
	drop eol;
run;

* La macro esegue lo studio di simulazione per tutti gli scenari selezionati
  dal dataset di partenza;
%macro simulation;

	%do iiii=1 %to &nscenari;
		%let id_scen=%scan(&lista_scenari,&iiii);
		data _NULL_;
			set scenari;
			if id_scen=&id_scen then do;
				call symput('n_a',n_a);
				call symput('n_b',n_b);
				call symput('lambda_A1',lambda_A1);
				call symput('lambda_A2',lambda_A2);
				call symput('lambda_B1',lambda_B1);
				call symput('lambda_B2',lambda_B2);
				call symput('t_admin',t_admin);
				call symput('theta_c',theta_c);
			end;
		run;
		
		* Inizializzazione di tre dataset che conterranno i p-values per i tre test eseguiti;
		%let scen=&id_scen;

		proc datasets library = results nolist;
			delete final_report&scen NPC_pvalues_scen&scen Pepemori_pvalues_scen&scen Gray_pvalues_scen&scen;
		run;
	

			%do rep=1 %to &n_sim;
				* necessario cancellare il ds con i p-value di Gray perch richiamato pi volte al di fuori di una procedura;
				proc datasets library = work nolist;
					delete output_Gray;
				run;
				DM 'clear log';
				data simul_data_scen&scen.rep&rep( keep= id_paz gruppo tempo tempo_cens evento );
					*call streaminit(4321 + &rep);
					mu_A1 = 1 / &lambda_A1;
					mu_A2 = 1 / &lambda_A2;
					mu_B1 = 1 / &lambda_B1;
					mu_B2 = 1 / &lambda_B2;
					pA = %sysevalf( &lambda_A1 /(&lambda_A1+ &lambda_A2) );
					pB = %sysevalf( &lambda_B1 /(&lambda_B1+ &lambda_B2) );
            		do id_paz = 1 to %eval(&n_A+&n_B);
						if id_paz le &n_A then do;
							gruppo = "A";
							tempo = (mu_A1+mu_A2) * rand("Exponential");
							evento = rand("Binomial", pA , 1);
							if evento = 0 then evento = 2;
							tempo_cens = 0 + (&theta_c-0)*rand("Uniform");
							if tempo > &t_admin or tempo_cens < tempo then evento = 0;
							output;
						end;
						else do;
							gruppo = "B";
							tempo = (mu_B1+mu_B2) * rand("Exponential");
							evento = rand("Binomial", pB , 1);
							if evento = 0 then evento = 2;
							tempo_cens = 0 + (&theta_c-0)*rand("Uniform");
							if tempo > &t_admin or tempo_cens < tempo then evento = 0;
							output;
						end;
					end;
				run;
      
			* Esecuzione del test di Gray per i due eventi col salvataggio della statistica test e del p-value
			  (ci sono due possibili esecuzioni a seconda della versione di SAS);
			%if &versione_sas = _9_4 %then %do;
				proc lifetest data = simul_data_scen&scen.rep&rep plots= &grafici;
					time tempo*evento(0)/ eventcode = 1 2;
					strata gruppo;
					ods output GrayTest = output_Gray;
				run;
			%end;
			%else %if &versione_sas = _9_3 %then %do;
				%CIF(data=simul_data_scen&scen.rep&rep, time=tempo, status=evento, group=gruppo, event=1, censored=0, alpha=.05, options=NOPLOT)
				data testresult; set testresult; Failcode = 1; run;
				proc append base = output_Gray data = testresult( rename = (test_stat = ChiSq pval = ProbChiSq df = DF) ); run;
				%CIF(data=simul_data_scen&scen.rep&rep, time=tempo, status=evento, group=gruppo, event=2, censored=0, alpha=.05, options=NOPLOT)
				data testresult; set testresult; Failcode = 2; run;
				proc append base = output_Gray data = testresult( rename = (test_stat = ChiSq pval = ProbChiSq df = DF) ); run;
			%end;
			proc append base = results.Gray_pvalues_scen&scen data = output_Gray; run;
      
			* Esecuzione del test di Pepe-Mori per i due eventi col salvataggio della statistica test e del p-value;
			%compcif(ds= simul_data_scen&scen.rep&rep, time= tempo, cens= evento, group= gruppo,val1= 'A',val2= 'B')
			quit; 
			
			data output_Pepemori; 
					keep failcode pvalue;
					set result; 
					failcode=1; 
			run;
			proc append base = results.Pepemori_pvalues_scen&scen data = output_Pepemori; run;

			data tmp;
				drop evento;
				set simul_data_scen&scen.rep&rep;
				nevento=evento;
				if evento=1 then nevento=2;
				if evento=2 then nevento=1;
			run;

			data tmp1;
				set tmp;
				rename nevento=evento;
				label nevento=evento;
			run;

			%compcif(ds= tmp1, time= tempo, cens= evento, group= gruppo,val1= 'A',val2= 'B')
			quit; 
			data output_Pepemori; 
					keep failcode pvalue;
					set result; 
					failcode=2; 
			run;
			proc append base = results.Pepemori_pvalues_scen&scen data = output_Pepemori; run;
			
			DM 'clear log';
			* Esecuzione della procedura NPC col salvataggio dei vari p-value;
			%NPC(ds=simul_data_scen&scen.rep&rep,tempo=tempo,evento=evento,gruppo=gruppo)
			proc append base = results.NPC_pvalues_scen&scen data = final_result_NPC; run;
		%end;
		
		/* Inseriamo in due variabili distinte i p-value del test di Gray */
		data Gray_pvalues_scen&scen._1;
			keep gray_pval1;
			set results.Gray_pvalues_scen&scen;
			if Failcode=1 then do; gray_pval1=ProbChiSq; output; end; 
		run;

		data Gray_pvalues_scen&scen._2;
			keep gray_pval2;
			set results.Gray_pvalues_scen&scen;
			if Failcode=2 then do; gray_pval2=ProbChiSq; output ; end; 
		run;

		data results.Gray_pvalues_scen&scen;
			merge Gray_pvalues_scen&scen._1 Gray_pvalues_scen&scen._2;
		run;
		
		DM 'clear log';
		/* Inseriamo in due variabili distinte i p-value del test di Pepe-Mori */
		data Pepemori_pvalues_scen&scen._1;
			keep Pepemori_pval1;
			set results.Pepemori_pvalues_scen&scen;
			if Failcode = 1 then do; Pepemori_pval1 = pvalue; output; end; 
		run;

		data Pepemori_pvalues_scen&scen._2;
			keep Pepemori_pval2;
			set results.Pepemori_pvalues_scen&scen;
			if Failcode = 2 then do; Pepemori_pval2 = pvalue; output; end; 
		run;

		data results.Pepemori_pvalues_scen&scen;
			merge Pepemori_pvalues_scen&scen._1 Pepemori_pvalues_scen&scen._2;
		run;
		
		DM 'clear log';
		* Creiamo il dataset col report definitivo;
		data assembla;
			merge results.Gray_pvalues_scen&scen results.NPC_pvalues_scen&scen results.Pepemori_pvalues_scen&scen;
		run;
		proc append base = results.final_report&scen data = assembla; run; 
	%end;


%mend simulation;



      
